/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-10-22 15:30:57.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.print.service.impl;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.core.print.common.jpa.SearchBuilder;
import cn.com.qmth.examcloud.core.print.common.jpa.SpecUtils;
import cn.com.qmth.examcloud.core.print.common.utils.Check;
import cn.com.qmth.examcloud.core.print.entity.ExamQuestionStructure;
import cn.com.qmth.examcloud.core.print.entity.ObjectiveQuestionStructure;
import cn.com.qmth.examcloud.core.print.entity.SubjectiveQuestionStructure;
import cn.com.qmth.examcloud.core.print.enums.QuesStructType;
import cn.com.qmth.examcloud.core.print.repository.ObjectiveQuestionStructureRepository;
import cn.com.qmth.examcloud.core.print.repository.SubjectiveQuestionStructureRepository;
import cn.com.qmth.examcloud.core.print.service.ExamQuestionStructureService;
import cn.com.qmth.examcloud.core.print.service.ExamStructureService;
import cn.com.qmth.examcloud.core.print.service.StatisticService;
import cn.com.qmth.examcloud.core.print.service.bean.examquestionstructure.ExamQuestionStructureInfo;
import cn.com.qmth.examcloud.core.print.service.bean.examstructure.ExamStructureInfo;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import static cn.com.qmth.examcloud.core.print.common.Constants.PRT_CODE_500;

/**
 * @author: fengdesheng
 * @since: 2018/10/17
 */
@Service
public class QuestionStructureServiceImpl implements ExamQuestionStructureService {
    private static final Logger log = LoggerFactory.getLogger(QuestionStructureServiceImpl.class);
    @Autowired
    private ObjectiveQuestionStructureRepository objectiveQuestionStructureRepository;
    @Autowired
    private SubjectiveQuestionStructureRepository subjectiveQuestionStructureRepository;
    @Autowired
    private ExamStructureService examStructureService;
    @Autowired
    private StatisticService statisticService;

    @Override
    public void savePaperQuestionStructure(@NotNull Long examId, @NotNull String paperId) {
        Check.isNull(examId, "考试ID不能为空！");
        Check.isEmpty(paperId, "试卷ID不能为空！");

        //从题库端获取试卷的试题结构信息
        ExamQuestionStructureInfo info = statisticService.findStructureByPaperId(examId, paperId);
        if (info == null) {
            log.warn("试卷试题结构不存在！ paperId is " + paperId);
            return;
        }

        //先清理旧数据
        objectiveQuestionStructureRepository.deleteByExamIdAndPaperId(examId, paperId);
        subjectiveQuestionStructureRepository.deleteByExamIdAndPaperId(examId, paperId);

        //保存客观题数据
        List<ObjectiveQuestionStructure> objectives = info.getObjectives();
        if (objectives != null && !objectives.isEmpty()) {
            for (ObjectiveQuestionStructure structure : objectives) {
                structure.setExamId(examId);
                structure.setPaperId(paperId);
            }
            objectiveQuestionStructureRepository.save(info.getObjectives());
        }

        //保存主观题数据
        List<SubjectiveQuestionStructure> subjectives = info.getSubjectives();
        if (subjectives != null && !subjectives.isEmpty()) {
            for (SubjectiveQuestionStructure structure : subjectives) {
                structure.setExamId(examId);
                structure.setPaperId(paperId);
            }
            subjectiveQuestionStructureRepository.save(info.getSubjectives());
        }
    }

    @Override
    public List<ObjectiveQuestionStructure> getObjectiveQuestionStructureList(Long orgId, Long examId, String paperId, String paperType) {
        Check.isNull(examId, "考试ID不能为空！");
        Check.isEmpty(paperId, "试卷ID不能为空！");

        SearchBuilder searches = new SearchBuilder()
                .eq("examId", examId)
                .eq("paperId", paperId);

        Specification<ObjectiveQuestionStructure> spec = SpecUtils.buildSearchers(ObjectiveQuestionStructure.class, searches.build());
        List<ObjectiveQuestionStructure> objectives = objectiveQuestionStructureRepository.findAll(spec);
        if (objectives == null || objectives.isEmpty()) {
            return Lists.newArrayList();
        }

        //获取考试结构设置的客观题数量
        ExamStructureInfo info = examStructureService.getExamStructure(orgId, examId);
        if (info == null || info.getQuestionStructure() == null) {
            throw new StatusException(PRT_CODE_500, "当前的考试结构设置不存在！");
        }

        //按试题类型封装试题集合
        Map<String, List<ObjectiveQuestionStructure>> structureMaps = new TreeMap<>();
        ExamQuestionStructure questionStructure = info.getQuestionStructure();
        for (ObjectiveQuestionStructure objective : objectives) {
            objective.setPaperType(paperType);//补全信息
            if (!structureMaps.containsKey(objective.getQuestionType())) {
                structureMaps.put(objective.getQuestionType(), Lists.newArrayList());
            }
            List<ObjectiveQuestionStructure> list = structureMaps.get(objective.getQuestionType());
            list.add(objective);
        }

        //补全阅卷时缺失的单选题
        this.fillQuestionStructure(structureMaps, QuesStructType.SINGLE_ANSWER_QUESTION, questionStructure.getSingleChoiceTotal());

        //补全阅卷时缺失的多选题
        this.fillQuestionStructure(structureMaps, QuesStructType.MULTIPLE_ANSWER_QUESTION, questionStructure.getMultipleChoiceTotal());

        //补全阅卷时缺失的判断题
        this.fillQuestionStructure(structureMaps, QuesStructType.BOOL_ANSWER_QUESTION, questionStructure.getBoolQuestionTotal());

        //返回所有客观题集合
        List<ObjectiveQuestionStructure> allObjectives = new ArrayList<>();
        for (String questionType : structureMaps.keySet()) {
            allObjectives.addAll(structureMaps.get(questionType));
        }
        return allObjectives;
    }

    private void fillQuestionStructure(Map<String, List<ObjectiveQuestionStructure>> structureMaps, QuesStructType quesStructType, final int total) {
        List<ObjectiveQuestionStructure> objectives = structureMaps.get(quesStructType.getTitle());
        final int size = objectives.size();

        if (total == 0) {
            //当前题型尚未定义试题数量，则置空当前集合
            structureMaps.put(quesStructType.getTitle(), Lists.newArrayList());
            return;
        }

        if (total < size) {
            //当前题型定义的试题数量“小于”当前试卷该题型的数量，则移除多余的项
            List<ObjectiveQuestionStructure> newList = new ArrayList<>();
            for (int i = 0; i < total; i++) {
                newList.add(objectives.get(i));
            }
            structureMaps.put(quesStructType.getTitle(), newList);
        } else if (total > size) {
            //当前题型定义的试题数量“大于”当前试卷该题型的数量，则补全剩余的项
            for (int i = (total - size); i < total; i++) {

                ObjectiveQuestionStructure obj = new ObjectiveQuestionStructure();
                obj.setUnitNum(size + i + 1);
                obj.setUnitScore(0D);
                obj.setAnswer("#");
                objectives.add(obj);
            }
        } else {
            //当前题型定义的试题数量“等于”当前试卷该题型的数量，则不用处理直接返回
            //ignore
        }
    }

    @Override
    public List<SubjectiveQuestionStructure> getSubjectiveQuestionStructureList(Long examId, String paperId, String paperType) {
        Check.isNull(examId, "考试ID不能为空！");
        Check.isEmpty(paperId, "试卷ID不能为空！");

        SearchBuilder searches = new SearchBuilder()
                .eq("examId", examId)
                .eq("paperId", paperId);

        Specification<SubjectiveQuestionStructure> spec = SpecUtils.buildSearchers(SubjectiveQuestionStructure.class, searches.build());
        List<SubjectiveQuestionStructure> subjectives = subjectiveQuestionStructureRepository.findAll(spec);
        if (subjectives == null || subjectives.isEmpty()) {
            return Lists.newArrayList();
        }

        for (SubjectiveQuestionStructure structure : subjectives) {
            structure.setPaperType(paperType);//补全信息
        }
        return subjectives;
    }

    @Override
    public void syncObjectiveCourseNameByCourseId(Long courseId, String courseName) {
        if (courseId == null || StringUtils.isBlank(courseName)) {
            return;
        }
        objectiveQuestionStructureRepository.updateCourseNameByCourseId(courseId, courseName);
    }

    @Override
    public void syncSubjectiveCourseNameByCourseId(Long courseId, String courseName) {
        if (courseId == null || StringUtils.isBlank(courseName)) {
            return;
        }
        subjectiveQuestionStructureRepository.updateCourseNameByCourseId(courseId, courseName);
    }

}